﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Collections;

namespace WindowsFormsApplication13
{
    public static class StringExtensions
    {
        /// <summary>
        /// Remove diacritics (accents) from string.
        /// </summary>
        public static string RemoveDiacritics(this string s)
        {
            string sFormD = s.Normalize(NormalizationForm.FormD);
            StringBuilder sb = new StringBuilder();

            foreach (char ch in sFormD)
            {
                UnicodeCategory uc = CharUnicodeInfo.GetUnicodeCategory(ch);
                if (uc != UnicodeCategory.NonSpacingMark)
                {
                    sb.Append(ch);
                }
            }

            return (sb.ToString().Normalize(NormalizationForm.FormC));
        }
    }

    public static class IEnumerableExtensions
    {
        /// <summary>
        /// Concatène deux listes (AddRange sur IEnumerable)
        /// </summary>
        /// <param name="source"></param>
        /// <param name="second"></param>
        /// <returns></returns>
        public static IEnumerable Append(this IEnumerable source,
                                            IEnumerable second)
        {
            foreach (var t in source) { yield return t; }
            foreach (var t in second) { yield return t; }
        }

        /// <summary>
        /// Permet de "Linquer" un foreach. L'action doit être une méthode.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="enumeration"></param>
        /// <param name="action">Méthode à exécuter pour chaque élément de l'énumérable</param>
        public static void ForEach<T>(this IEnumerable<T> enumeration, Action<T> action)
        {
            foreach (T item in enumeration)
            {
                action(item);
            }
        }
    }

}
