﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinTree;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win;

namespace WindowsFormsApplication13
{
    public partial class PHDropDownTreeView : UserControl, IProvidesEmbeddableEditor
    {
        private Infragistics.Win.ValueList _ds = new Infragistics.Win.ValueList();
        private IList<UltraTreeNode> _orderedList = new List<UltraTreeNode>();

        #region IProvidesEmbeddableEditor Members

        [Browsable(false)]
        public EmbeddableEditorBase Editor
        {
            get { return this.ultraComboEditor1.Editor; }
        }

        #endregion

        #region UI Events

        public PHDropDownTreeView()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.phTreeView1.MaximumSize = new Size(this.GetMaximumWidth(), int.MaxValue);
            (this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton).RightAlignDropDown =
                this.Left - this.phTreeView1.Width < 0 ? DefaultableBoolean.False : DefaultableBoolean.True;
        }

        private int GetMaximumWidth()
        {
            if (this.ParentForm != null)
            {
                return (int)(this.ParentForm.ClientSize.Width * 0.40f) < this.phTreeView1.Width ? this.phTreeView1.Width : (int)(this.ParentForm.ClientSize.Width * 0.40f);
            }
            else
            {
                return this.phTreeView1.Width;
            }
        }

        private int GetRequiredWidth()
        {
            SizeF size = this.Size;
            List<String> texts = new List<string>();
            foreach (var node in this.TreeView.Nodes)
            {
                this.GetNodeTexts(node, "", texts);
            }
            foreach (var item in texts)
            {
                using (var g = this.CreateGraphics())
                {
                    var tSize = g.MeasureString(item, this.TreeView.Font);
                    size.Width = tSize.Width > size.Width ? (tSize.Width + 20) : size.Width;
                }
            }
            return (int)size.Width;
        }

        private void ultraComboEditor1_BeforeEditorButtonDropDown(object sender, BeforeEditorButtonDropDownEventArgs e)
        {
            var reqWidth = this.GetRequiredWidth();
            this.phTreeView1.Width = this.Width > reqWidth ? this.Width : reqWidth;
            if (_readOnly == true)
            {
                e.Cancel = true;
                return;
            }
            if (PHBeforeEditorButtonDropDown != null)
            {
                PHBeforeEditorButtonDropDown(this, e);
            }
        }

        private void ultraComboEditor1_AfterEditorButtonCloseUp(object sender, Infragistics.Win.UltraWinEditors.EditorButtonEventArgs e)
        {
            if (PHAfterEditorButtonCloseUp != null)
            {
                PHAfterEditorButtonCloseUp(this, e);
            }
            else
            {
                UpdateText();
            }
        }

        private void ultraComboEditor1_BeforeDropDown(object sender, CancelEventArgs e)
        {
            e.Cancel = true;
            if (!_readOnly)
            {
                DropDownEditorButton dd = this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton;
                this.DropDown();
            }
        }

        private void ultraComboEditor1_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
            if (char.IsLetterOrDigit(e.KeyChar))
            {
                DropDownEditorButton dd = this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton;
                if (dd != null && dd.IsDroppedDown)
                {
                    e.Handled = true;
                    return;
                }
                this.DropDown();
                SelectFirstItemInTree(this.phTreeView1.TreeView.Nodes, e.KeyChar.ToString());
            }
        }

        private void DropDown()
        {
            DropDownEditorButton dd = this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton;
            if (dd != null && dd.IsDroppedDown == false)
            {
                dd.DropDown();
            }
        }

        private void ultraTree1_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (e.KeyCode == Keys.Tab)
            {
                DropDownEditorButton dd = this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton;
                dd.CloseUp();
                this.phTreeView1.TreeView.SelectNextControl(this, true, true, false, true);
            }
        }

        private void PHDropDownTreeview2_Resize(object sender, EventArgs e)
        {
            this.phTreeView1.Width = this.Width;
            this.ultraComboEditor1.Width = this.Width;
        }

        private void ultraComboEditor1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                this.ClearSelection();
                e.Handled = true;
            }
        }

        private void phTreeView1_PHBeforeExpand(object sender, CancelableNodeEventArgs e)
        {
            if (PHBeforeExpand != null)
            {
                PHBeforeExpand(this, e);
            }
        }

        private void GetNodeTexts(UltraTreeNode node, string parentText, List<String> texts)
        {
            string s = parentText + (node.Text.Length > 1 ? node.Text.Substring(1) : node.Text);
            texts.Add(s);
            foreach (var child in node.Nodes)
            {
                this.GetNodeTexts(child, s, texts);
            }
        }

        private void phTreeView1_PHKeyPress(object sender, KeyPressEventArgs e)
        {
            if (char.IsLetterOrDigit(e.KeyChar))
            {
                this.phTreeView1.TreeView.SelectedNodes.Clear();
                SelectFirstItemInTree(this.phTreeView1.TreeView.Nodes, e.KeyChar.ToString());
                e.Handled = true;
            }
        }

        private void phTreeView1_PHBeforeSelect(object sender, BeforeSelectEventArgs e)
        {
            if (PHBeforeSelect != null)
            {
                PHBeforeSelect(sender, e);
            }
        }

        private void phTreeView1_PHAfterSelect(object sender, SelectEventArgs e)
        {
            if (PHAfterSelect != null)
            {
                PHAfterSelect(this, e);
            }
        }

        private void ultraComboEditor1_ValueChanged(object sender, EventArgs e)
        {
            if (ValueChanged != null)
            {
                ValueChanged(sender, e);
            }
        }

        #endregion

        #region Private functions

        private void LoadNodes(UltraTreeNode node)
        {
            _ds.ValueListItems.Add(node);
            if (node.HasNodes)
            {
                foreach (var child in node.Nodes)
                {
                    LoadNodes(child);
                }
            }
            var reqWidth = this.GetRequiredWidth();
            this.phTreeView1.Width = this.Width > reqWidth ? this.Width : reqWidth;
        }

        private void SelectFirstItemInTree(TreeNodesCollection nodes, string searchValue)
        {
            foreach (var node in nodes)
            {
                var text = node.Text.RemoveDiacritics().ToLower();
                var searchText = searchValue.RemoveDiacritics().ToLower();

                if (text.StartsWith(searchText))
                {
                    this.phTreeView1.TreeView.SelectedNodes.Clear();
                    this.phTreeView1.TreeView.ActiveNode = node;
                    node.Selected = true;
                    break;
                }
                else
                {
                    SelectFirstItemInTree(node.Nodes, searchValue);
                }
            }
        }

        private void UpdateText()
        {
            ultraComboEditor1.Items.Clear();
            _orderedList.Clear();

            if (this.phTreeView1.TreeView.SelectedNodes.Count > 0)
            {
                var str = string.Empty;

                foreach (var item in phTreeView1.TreeView.Nodes)
                {
                    AddNodeToList(item);
                }

                if (phTreeView1.SelectSimilarItem) // For UT5.2.12 and SE5.2.50
                {
                    str = string.Join(", ", _orderedList.Select(n => n.Text).Distinct().ToArray());
                }
                else
                {
                    str = string.Join(", ", _orderedList.Select(n => n.Text).ToArray());
                }

                ultraComboEditor1.Items.Add(str);
                ultraComboEditor1.SelectedItem = ultraComboEditor1.Items[0];
            }
            else
            {
                ultraComboEditor1_ValueChanged(this, new EventArgs());
            }
        }

        private void AddNodeToList(UltraTreeNode node)
        {
            if (node.Selected && !_orderedList.Contains(node))
            {
                if (TextFilter == TreeViewTypeFilter.None)
                {
                    _orderedList.Add(node);
                }
                else
                {
                    var textFilter = "organisation_";
                    if (node.Key.Contains(textFilter))
                    {
                        _orderedList.Add(node);
                    }
                }
            }

            foreach (var item in node.Nodes)
            {
                AddNodeToList(item);
            }
        }

        #endregion

        #region Exposed Properties

        public UltraComboEditor ComboEditor
        {
            get { return this.ultraComboEditor1; }
        }

        [Description("Infragistics UltraTree control")]
        [Browsable(false)]
        public UltraTree TreeView
        {
            get { return this.phTreeView1.TreeView; }
        }

        [Description("Set the selection mode: All, Single, Multiple")]
        [DefaultValue(SelectionModes.Single)]
        public SelectionModes SelectionMode
        {
            get
            {
                return this.phTreeView1.SelectionMode;
            }
            set
            {
                this.phTreeView1.SelectionMode = value;
            }
        }

        [Description("")]
        [Browsable(true)]
        [DefaultValue(TreeViewTypeFilter.None)]
        public TreeViewTypeFilter TextFilter { get; set; }

        [EditorBrowsable(EditorBrowsableState.Always),
        DefaultValue(""),
        Browsable(true),
        DesignerSerializationVisibility(DesignerSerializationVisibility.Visible),
        Bindable(true)]
        public override string Text
        {
            get { return this.ultraComboEditor1.Text; }
        }

        private bool _readOnly;
        [EditorBrowsable(EditorBrowsableState.Always),
        DefaultValue(false),
        Browsable(true),
        DesignerSerializationVisibility(DesignerSerializationVisibility.Visible),
        Bindable(true)]
        public bool ReadOnly
        {
            get
            {
                return _readOnly;
            }
            set
            {
                _readOnly = value;
                this.ultraComboEditor1.ReadOnly = _readOnly;
            }
        }

        public bool AlignLeft
        {
            get
            {
                return (this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton).RightAlignDropDown == DefaultableBoolean.True;
            }
            set
            {
                (this.ultraComboEditor1.ButtonsRight[0] as DropDownEditorButton).RightAlignDropDown =
                    value ? DefaultableBoolean.False : DefaultableBoolean.True;
            }
        }

        public bool SelectSimilarItem
        {
            get
            {
                return phTreeView1.SelectSimilarItem;
            }
            set
            {
                phTreeView1.SelectSimilarItem = value;
            }
        }

        public bool ForFilterSort
        {
            get
            {
                return phTreeView1.ForFilterSort;
            }
            set
            {
                phTreeView1.ForFilterSort = value;
            }
        }

        #endregion

        #region Exposed Events

        [Category("PrimHeure")]
        public event EditorButtonEventHandler PHAfterEditorButtonCloseUp;

        [Category("PrimHeure")]
        public event BeforeEditorButtonDropDownEventHandler PHBeforeEditorButtonDropDown;

        [Category("PrimHeure")]
        public event EventHandler<CancelableNodeEventArgs> PHBeforeExpand;

        [Category("PrimHeure")]
        public event BeforeNodeSelectEventHandler PHBeforeSelect;

        [Category("PrimHeure")]
        public event AfterNodeSelectEventHandler PHAfterSelect;

        [Category("PrimHeure")]
        public event EventHandler ValueChanged;

        #endregion

        #region Public Functions

        public void SetValue(string nodeKey)
        {
            var node = this.phTreeView1.TreeView.GetNodeByKey(nodeKey);

            this.phTreeView1.TreeView.ActiveNode = node;
            node.Selected = true;
            UpdateText();
        }

        public void ClearSelection()
        {
            TreeView.SelectedNodes.Clear();
            UpdateText();
        }

        #endregion
    }
}
